//$Id: ufl.hpp,v 1.2 2003/12/03 01:35:55 magh Exp $

/* ------------------------------------------------------------------------
 Author(s): Matthew Galati (magh@lehigh.edu)
            Yan Xu         (yax2@lehigh.edu)

 (c) Copyright 2003 Lehigh University. All Rights Reserved.

 This software is licensed under the Common Public License. Please see 
 accompanying file for terms.    
---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------*/
#ifndef UFL_HPP
#define UFL_HPP

#include <iostream>
using namespace std;

#include "OsiSolverInterface.hpp"
#include "OsiCuts.hpp"

const double tolerance = 0.0001;
/*---------------------------------------------------------------------*/
class UFL{
public:
  UFL(const char* datafile);
  ~UFL();
  void create_model();
  void solve_model(ostream *os = &cout);
  double simulate_root(ostream *os = &cout);

private:
  OsiSolverInterface * si;
  double * trans_cost; //c[i][j] -> c[xindex(i,j)]
  double * fixed_cost; //f[j]
  double * demand;     //d[j]
  int M;               //number of clients (index on i)
  int N;               //number of depots  (index in j)
  double total_demand; //sum{j in N} d[j]
  int *integer_vars;

  inline int xindex(const int i, const int j) {return i*N + j;}
  inline int yindex(const int j)              {return M*N + j;}

private:
  void read_data(const char* datafile);
  void print_solution(const double * solution, 
		      const double * objCoeff, 
		      ostream * os = &cout);
  double find_upperbound();
  int checkAddCut(OsiCuts & cutPool, const OsiRowCut & cut);
  int checkAddCuts(OsiCuts & cutPool, OsiCuts & cuts);
  int removeIneffCuts();
  int generate_cuts(OsiCuts & cutPool);
  int generate_cuts_CGL(OsiCuts & cutPool, ostream * os = &cout);
  OsiSolverInterface* initialize_solver();
};

#endif
